// Contains class definitions
// Copyright (C) 1997 by John P. Weiss
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#include "AudioPanel.h"
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <qbitmap.h>
#include <qimage.h>
#include <string.h>



// ***************
//  Here, we define the actual functions



//||||||||||||||||||||||||||||||||||||||||||||||||
//
//  Class AudioPanel
//
//||||||||||||||||||||||||||||||||||||||||||||||||



// Constructor for the AudioPanel object.
//
AudioPanel::AudioPanel( QWidget *parent, char *dn, bool fullsize, 
				    bool compact, bool useEject ) 
  : QWidget( parent, "", WStyle_Customize | WStyle_DialogBorder )
{
  QColor displayC;
  QFont panelF, infotextF;
  float scale;
  
  //Define the size
  if(fullsize)
	{
	scale = 1.0;
	panelF = QFont(MEDFONT);
	infotextF = QFont(BIGFONT);
	}
  else
	{
	scale = SCALE_SIZE_CUTOFF;
	panelF = QFont(SMALLFONT);
	infotextF = QFont(MEDFONT);
	}

  itsParent = parent;
  displayC = COLOR_DISPLAY;
  hasIcon = false;
  theDevPtr = (AudioDev *)0;
  if(dn == (char *)0)
	default_name = "panel";
  else
	default_name = dn;


  // Create the children.  All children set their palette and main font
  // from this widget's, except for pDisplay.
  aud = new pAudio( this, scale );
  ctrl = new pControl( this, scale, useEject );
  spec = new pSpecial( this, compact );
  disp = new pDisplay( this, displayC, infotextF, scale, compact );

  // Set the default colors and fonts.
  setDisplayColor( displayC );
  changeFont( panelF );


  //Position them.
  disp->moveAll( ctrl->xEndPt() );
  aud->moveAll( disp->xEndPt() );
  spec->moveAll( (aud->xEndPt()).x() - spec->width(),
		    (ctrl->yEndPt()).y() - spec->height() );

  //Now do some settings on the main window.
  resize( (aud->xEndPt()).x(), (ctrl->yEndPt()).y() );
  setFixedSize( size() );
  pluginDefault();
}


// Destructor for the AudioPanel object.
//
AudioPanel::~AudioPanel( void )
{
   delete aud;
   delete ctrl;
   delete spec;
   delete disp;
}


// Performs default signal-slot connections.
//
void AudioPanel::pluginDefault( void )
{
  connect( ctrl, SIGNAL(titlebarOn(bool)),
		 SLOT(toggleWTB(bool)) );
  connect( ctrl->loadejectb, SIGNAL(clicked()),
		 SLOT(doLoadEject()) );
  connect( ctrl->powerb, SIGNAL(clicked()),
		 SLOT(doPower()) );
  connect( ctrl->iconifyb, SIGNAL(clicked()),
		 SLOT(doIconify()) );
  connect( ctrl, SIGNAL(pressed(QPoint)),
		 SLOT(pressedAt(QPoint)) );
  connect( ctrl, SIGNAL(moving(QPoint)),
		 SLOT(doMove(QPoint)) );

  connect( spec, SIGNAL(pressed(QPoint)),
		 SLOT(pressedAt(QPoint)) );
  connect( spec, SIGNAL(moving(QPoint)),
		 SLOT(doMove(QPoint)) );

  connect( spec, SIGNAL(newTimeMode(int)),
		 disp, SLOT(newTimeMode(int)) );
  connect( spec, SIGNAL(newPlayMode(int)),
		 disp, SLOT(newPlayMode(int)) );

  connect( disp, SIGNAL(timePressed(QPoint)),
		 spec, SLOT(tmenuOpen(QPoint)) );
  connect( disp, SIGNAL(modePressed(QPoint)),
		 spec, SLOT(pmenuOpen(QPoint)) );

  connect( disp, SIGNAL(titleReleased()),
		 SLOT(swapTitleArtist()) );
  connect( disp, SIGNAL(pressed(QPoint)),
		 SLOT(pressedAt(QPoint)) );
  connect( disp, SIGNAL(moving(QPoint)),
		 SLOT(doMove(QPoint)) );

  emit disp->newTimeMode(tmElapsed);
}


// Connects the pAudio object to external signals from an audio
// device. 
//
void AudioPanel::pluginAudioControl( AudioDev *dev )
{
  connect( aud->playpauseb, SIGNAL(clicked()),
		 dev, SLOT(StartPlayOrPause()) );
  connect( aud->stopb, SIGNAL(clicked()),
		 dev, SLOT(StopPlay()) );
  connect( aud->nextb, SIGNAL(clicked()),
		 dev, SLOT(Next()) );
  connect( aud->prevb, SIGNAL(clicked()),
		 dev, SLOT(Prev()) );
  connect( aud->ffb, SIGNAL(pressed()),
		 dev, SLOT(StartScanFwd()) );
  connect( aud->rewb, SIGNAL(pressed()),
		 dev, SLOT(StartScanRev()) );
  connect( aud->ffb, SIGNAL(released()),
		 dev, SLOT(StopScan()) );
  connect( aud->rewb, SIGNAL(released()),
		 dev, SLOT(StopScan()) );
  connect( spec->vol_upb, SIGNAL(clicked()),
		 dev, SLOT(IncVol()) );
  connect( spec->vol_dnb, SIGNAL(clicked()),
		 dev, SLOT(DecVol()) );
}


// Connects the pDisplay object to external slots from an audio
// device. 
// 
void AudioPanel::pluginDisplayChange( AudioDev *dev )
{
  isSwapped = false;
  theDevPtr = dev;
  disp->useTitleLabel();

  connect( dev, SIGNAL(newVolume(float)),
		 disp, SLOT(newVolume(float)) );
  connect( dev, SIGNAL(newTime(int)),
		 disp, SLOT(newTime(int)) );
  connect( dev, SIGNAL(newItemNumber(int)),
		 disp, SLOT(newTrack(int)) );
  connect( dev, SIGNAL(newItemLabel(char *)),
		 disp, SLOT(newTrackName(char *)) );
  connect( dev, SIGNAL(newProgLabel(char *)),
		 disp, SLOT(newTitleName(char *)) );
  connect( dev, SIGNAL(newPlayState(int)),
		 disp, SLOT(newPlayMode(int)) );

  connect( spec, SIGNAL(newTimeMode(int)),
		 dev, SLOT(newTimeMode(int)) );
  connect( spec, SIGNAL(newPlayMode(int)),
		 dev, SLOT(newPlayMode(int)) );

  connect( dev, SIGNAL(newProgArtist(char *)),
		 SLOT(newWinTitle(char *)) );
}


// Connects the pSpecial object to various dialog boxes
// 
void AudioPanel::pluginPanels( APDialog *prog_dlg, 
						 APDialog *config_dlg,
						 APDialog *playlst_dlg )
{
   if(prog_dlg == NULLDLG)
	 prog_dlg = &nullmesg;

   if(config_dlg == NULLDLG)
	 config_dlg = &nullmesg;

   if(playlst_dlg == NULLDLG)
	 playlst_dlg = &nullmesg;

   connect( spec->progb, SIGNAL(clicked()),
		  prog_dlg, SLOT(open()) );
   connect( spec->configb, SIGNAL(clicked()),
		  config_dlg, SLOT(open()) );
   connect( spec->playlstb, SIGNAL(clicked()),
		  playlst_dlg, SLOT(open()) );
}


// Private slot for emitting a lockable eject signal.
//
void AudioPanel::doLoadEject(void)
{
  if(!(ctrl->isLocked))
	{
	emit loadeject();
	}
}


// Private slot for emitting a lockable quit signal.
void AudioPanel::doPower(void)
{
  if(!(ctrl->isLocked))
	{
	emit quit();
	}
}


// Private slot for iconifying the window.
void AudioPanel::doIconify(void)
{
  iconify();
}


// Private slot for changing the window title
void AudioPanel::newWinTitle(char *nutxt)
{
   if (nutxt == 0)
	 nutxt = default_name;

   setIconText(nutxt);
   setCaption(nutxt);
}


// Private slot for swapping the connections to the title display with
// that to the artist name.
void AudioPanel::swapTitleArtist(void)
{
   if(theDevPtr == (AudioDev *)0)
	 return;

   if(isSwapped)
	 {
	 isSwapped=false;
	 disp->useTitleLabel();
	 disconnect( theDevPtr, SIGNAL(newProgLabel(char *)),
			   0, 0 );
	 disconnect( theDevPtr, SIGNAL(newProgArtist(char *)),
			   0, 0 );
	 connect( theDevPtr, SIGNAL(newProgLabel(char *)),
			disp, SLOT(newTitleName(char *)) );
	 connect( theDevPtr, SIGNAL(newProgArtist(char *)),
			SLOT(newWinTitle(char *)) );
	 }
   else
	 {
	 isSwapped=true;
	 disp->useArtistLabel();
	 disconnect( theDevPtr, SIGNAL(newProgLabel(char *)),
			   0, 0 );
	 disconnect( theDevPtr, SIGNAL(newProgArtist(char *)),
			   0, 0 );
	 connect( theDevPtr, SIGNAL(newProgLabel(char *)),
			SLOT(newWinTitle(char *)) );
	 connect( theDevPtr, SIGNAL(newProgArtist(char *)),
			disp, SLOT(newTitleName(char *)) );
	 }

   theDevPtr->cleanDisplay();
}


// Private slot for storing the mouse position of a press event.
//
void AudioPanel::pressedAt(QPoint wp)
{
   pressOffset = wp-pos();
}


// Private slot for performing a move.  The position is in global
// coordinates.
//
void AudioPanel::doMove(QPoint wp)
{
   move(wp-pressOffset);
}


// Private slot for toggling the presence of a window-manager title
// bar. 
//
void AudioPanel::toggleWTB(bool makeWTB)
{
  QPoint cur_pos;
  char *oldtitle;
  
  cur_pos = pos();
  oldtitle = strdup(caption());

  if(makeWTB)
	{
	recreate( itsParent, WStyle_Customize | WStyle_NormalBorder,
			cur_pos, true );
	}
  else
	{
	recreate( itsParent, WStyle_Customize | WStyle_DialogBorder,
			cur_pos, true );
	}
  setCaption(oldtitle);
  makePanelIcon();
  delete oldtitle;
}


// Sets the icon_xpm_text pointer.
//
void AudioPanel::setPanelIcon(const char *xpm0[])
{
   QImage iqi( xpm0 );
   iqi.setAlphaBuffer(true);
   icon_px = iqi;
   hasIcon = true;
   
   makePanelIcon();
}


// Overloaded version of previous method.
//
void AudioPanel::setPanelIcon(QPixmap& ipix)
{
   icon_px = ipix;
}


// Overloaded version of previous method.
//
void AudioPanel::setPanelIcon(QImage& iimg)
{
   icon_px = iimg;
}


// Sets the panel's icon from the default icon_xpm_text pointer.
//
void AudioPanel::makePanelIcon(void)
{
   if(!hasIcon)
	 return;

   setIcon( icon_px );

   // Xlib stuff to set the icon mask, purloined directly from the Qt
   // source code and modified to suit our own evil purposes!
   XWMHints *wm_hints;      // window manager hints
   wm_hints = XGetWMHints( x11Display(), winId() );
   wm_hints->icon_mask = (icon_px.mask())->handle();
   wm_hints->flags |= IconMaskHint;
   XSetWMHints( x11Display(), winId(), wm_hints );
   XFree( wm_hints );
   return;
}


// Reimplemented virtual function to receive a mouse press event.  It
// stores the position at which the mouse is pressed.
//
void AudioPanel::mousePressEvent( QMouseEvent *me )
{
   pressOffset = mapToGlobal(me->pos()) - pos();
}


// Reimplemented virtual function to receive a mouse move event.  It
// calls doMove() to move the entire panel.
//
void AudioPanel::mouseMoveEvent( QMouseEvent *me )
{
   doMove( mapToGlobal(me->pos()) );
}




//||||||||||||||||||||||||||||||||
//
// End
//
